import c4d
from c4d import gui


#---------------------------------------------------------------------------
#---- SELECT PARENT OBJECT with a MIX MATERIAL NETWORK------
#---- Script will duplicate all connected materials ------
#---- And modify path of imagetexture which is connected to amount slot ----
#---- Coded by Ahmet Oktar with idea of Yvan Feusi  
#---------------------------------------------------------------------------

def collectShaders(tx, shaders):
    while tx:
        shaders.append(tx)
        if(tx.GetDown()): collectShaders(tx.GetDown(), shaders)
        tx=tx.GetNext()
        
#--- duplicate mixmat and connected mixmats
def cloneMixMat(mixmat, newmats):
    m = mixmat.GetClone()  #clone selected material
    doc.InsertMaterial(m)     #insert to document
    newmats.append(m)
    tx = m[c4d.MIXMATERIAL_AMOUNT_LNK]
    m1 = m[c4d.MIXMATERIAL_TEXTURE1]
    m2 = m[c4d.MIXMATERIAL_TEXTURE2]
    doc.AddUndo(c4d.UNDOTYPE_NEW, m)
        
    #tx[c4d.IMAGETEXTURE_FILE]="A-4mb.jpg"    #change imagetexture filename
    
    if m1!=None and m1.GetType() == 1029622:
        m[c4d.MIXMATERIAL_TEXTURE1] = cloneMixMat(m1, newmats)
    
    if m2!=None and m2.GetType() == 1029622:
        m[c4d.MIXMATERIAL_TEXTURE2] = cloneMixMat(m2, newmats)

    return m    

#--- assign material to object
def assignMat(obj, mat):
    t = obj.GetTag(c4d.Ttexture)
    if t==None:
        t = c4d.BaseTag(c4d.Ttexture)
        obj.InsertTag(t)
    t[c4d.TEXTURETAG_MATERIAL]=mat
    t[c4d.TEXTURETAG_PROJECTION]=6
    
#--- count suffix from end as <cnt> number
def findSuffixPos(str, FX, cnt):
    n=len(str)-1
    c=0
    n1=-1
    while n>0:
        if str[n]==FX:
            c=c+1
            if c==cnt: n1=n
        n = n-1;
        
    n = str.find(FX, n+2)
    return n1
    
#--- entry function ----
def main():
    topobj = doc.GetActiveObject() 
    if topobj==None: gui.MessageDialog("Select parent of objects."); return
    
    tag = topobj.GetTag(c4d.Ttexture)
    if tag==None:  gui.MessageDialog("Assing one mix material to selected object."); return

    mat = tag[c4d.TEXTURETAG_MATERIAL]
    if mat.GetType()!=1029622: gui.MessageDialog("Assing one mix material to selected object."); return
    
    doc.StartUndo()
    obj=topobj.GetDown()
    
    #--- iterate all child objs, clone mats, change image paths
    while obj!=None:
        newmats =[]
        oname = obj.GetName()
        
        n = findSuffixPos(oname,"_", 2)
        #print "OBJ:"+oname+"  "+oname[n:]
        mix = cloneMixMat(mat, newmats)
        assignMat(obj, mix)
        
        for m in newmats:
            #print "   MAT:",m.GetName()
            shaders = []
            collectShaders(m.GetFirstShader(), shaders)
            for tx in shaders:
                if(tx.GetType()==1029508): 
                    path = tx[c4d.IMAGETEXTURE_FILE]
                    n2 = findSuffixPos(path,"_", 2)
                    n3 = findSuffixPos(path,".", 1)
                    #print "         "+tx.GetName()+"  "+path[:n2]+oname[n:]+path[n3:]
                    tx[c4d.IMAGETEXTURE_FILE] = path[:n2]+oname[n:]+path[n3:]
            
        obj=obj.GetNext()
                
    
    doc.EndUndo()
    c4d.EventAdd()

if __name__=='__main__':
    main()
